﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using Microsoft.Web.Media.SmoothStreaming;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.Windows.Markup;
using System.Globalization;
using System.Collections.Generic;
using System.Windows.Browser;

namespace SSPlayerControls
{
  
  public class PlayerUI : Control, INotifyPropertyChanged
  {
    Scrubber scrubber = null;
    ButtonsPanel buttonsPanel = null;
    Grid MediaElementContainer = null;
    internal SmoothStreamingMediaElement ssme = null; 



    public double FrameRate
    {
      get { return (double)GetValue(FrameRateProperty); }
      set { SetValue(FrameRateProperty, value); }
    }

    // Using a DependencyProperty as the backing store for FrameRate.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty FrameRateProperty =
        DependencyProperty.Register("FrameRate", typeof(double), typeof(PlayerUI), new PropertyMetadata(23D));
    public string SmoothSource
    {
      get { return (string)GetValue(SmoothSourceProperty); }
      set { SetValue(SmoothSourceProperty, value); }
    }

    // Using a DependencyProperty as the backing store for SmoothSource.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty SmoothSourceProperty =
        DependencyProperty.Register("SmoothSource", typeof(string), typeof(PlayerUI), new PropertyMetadata(null, new PropertyChangedCallback(OnSmoothSourceChanged)));

    //Change handler for dependency property SmoothSourceProperty
    private static void OnSmoothSourceChanged(DependencyObject Src, DependencyPropertyChangedEventArgs Args)
    {
      PlayerUI thisObj = Src as PlayerUI;
      //act on the change...
      if (thisObj.ssme != null && Uri.IsWellFormedUriString(Args.NewValue as string, UriKind.Absolute))
      {
        thisObj.SetMediaSource(Args.NewValue as string, thisObj.ssme);
      }
    }

    private void SetMediaSource(string MediaSourceUri,
      SmoothStreamingMediaElement ssme)
    {

      if (MediaSourceUri.Contains(".ism"))
        ssme.SmoothStreamingSource = new Uri(MediaSourceUri);
      else
        ssme.Source = new Uri(MediaSourceUri);
    }  

   

    public PlayerUI()
    {
      base.DefaultStyleKey = typeof(PlayerUI);
    }

    public override void OnApplyTemplate()
    {
      base.OnApplyTemplate();
      ssme = GetTemplateChild("ssme") as SmoothStreamingMediaElement;
      scrubber = GetTemplateChild("scrubber") as Scrubber;
      buttonsPanel = GetTemplateChild("buttonsPanel") as ButtonsPanel; 

      MediaElementContainer = GetTemplateChild("MediaElementContainer") as Grid;
      if (scrubber != null && ssme != null)
        scrubber.Viewer = ssme;
      if (buttonsPanel != null && ssme != null)
      {
        buttonsPanel.Viewer = ssme;
        //the position change actually happens in the Scrubber class(Scrubber.cs) - line 201
        buttonsPanel.FrameBackward += new EventHandler((s, e) =>
        {
          scrubber.Value -= (double)((TimeCode.FromFrames(1, TimeCode.ParseFramerate(FrameRate))).TotalSecondsPrecision * 1000); // - you could also try this - has the same effect
        });
        buttonsPanel.FrameForward += new EventHandler((s, e) =>
        {
          scrubber.Value += (double)((TimeCode.FromFrames(1, TimeCode.ParseFramerate(FrameRate))).TotalSecondsPrecision * 1000);// - you could also try this - has the same effect
        });

      }

      if (ssme != null)
      {

        ssme.ConfigPath = "config.xml";

        ssme.MediaOpened += new RoutedEventHandler((sender, args) =>
        {
          
          ssme.Position = TimeSpan.FromTicks((long)(0));
          ScheduleClips();
         
          
        });
        ssme.ClipClickThrough += new EventHandler<ClipEventArgs>((s, e) =>
        {
          HtmlPage.Window.Invoke("onClickThruRequest", e.Context.ClipInformation.ClickThroughUrl.AbsoluteUri);
          return;
        });
        ssme.ClipError += new EventHandler<ClipEventArgs>((s, e) =>
        {
          return;
        });
        ssme.ClipStateChanged += new RoutedEventHandler((s, e) =>
        {
          return;
        });
        ssme.ClipProgressUpdate += new EventHandler<ClipPlaybackEventArgs>((s, e) =>
        {
          System.Diagnostics.Debug.WriteLine("Clip Position - {0}", e.Progress.ToString());
          return;
        });
       
        
      }


    }


    /** UNCOMMENT TO Schedule clips in a chained fashion **/

    //private void ScheduleClips()
    //{
    //  StreamInfo siAdClips = ssme.AvailableStreams.Where(si => si.Name.ToLower() == "adclips").
    //      FirstOrDefault();
    //  if (siAdClips != null && siAdClips.AvailableTracks.Count > 0)
    //  {
    //    ClipContext clipCtx = null;
    //    foreach (TimelineEvent te in siAdClips.AvailableTracks[0].TrackData)
    //    {
    //      XElement xeClipData = XElement.Parse(Encoding.UTF8.GetString(te.EventData, 0,
    //          te.EventData.Length));
    //      //if this is the first clip to be scheduled
    //      if (clipCtx == null)
    //      {
    //        clipCtx = ssme.ScheduleClip(
    //            new ClipInformation
    //            {
    //              ClickThroughUrl = new Uri(xeClipData.Attribute("ClickThruUri").Value),
    //              ClipUrl = new Uri(xeClipData.Attribute("Uri").Value),
    //              IsSmoothStreamingSource = xeClipData.Attribute("Uri").Value.
    //              ToLower().Contains("ism") ? true : false,
    //              Duration = TimeSpan.Parse(xeClipData.Attribute("Duration").Value)
    //            },
    //            te.EventTime, //pass in the start time for the clip
    //            true,
    //            null);
    //      }
    //      else //subsequent clips
    //      {
    //        clipCtx = ssme.ScheduleClip(
    //            new ClipInformation
    //            {
    //              ClickThroughUrl = new Uri(xeClipData.Attribute("ClickThruUri").Value),
    //              ClipUrl = new Uri(xeClipData.Attribute("Uri").Value),
    //              IsSmoothStreamingSource = xeClipData.Attribute("Uri").Value.
    //              ToLower().Contains("ism") ? true : false,
    //              Duration = TimeSpan.Parse(xeClipData.Attribute("Duration").Value)
    //            },
    //            clipCtx, //pass in the clip context for the previous scheduled clip to chain
    //            true,
    //            null);
    //      }
    //    }

    //    ssme.ClipMediaElementStyle = this.Resources["ClipStyle"] as Style;
    //  }
    //}

    /** Schedule clips at absolute timepoints **/
    private void ScheduleClips()
    {
      ssme.ClipPreloadTime = TimeSpan.Parse("00:00:20");
      //get the clip data stream
      StreamInfo siAdClips = ssme.AvailableStreams.Where(si => si.Name.ToLower() == "adclips").FirstOrDefault();
      //if we have tracks
      if (siAdClips != null && siAdClips.AvailableTracks.Count > 0)
      {
        //for each event in that track
        foreach (TimelineEvent te in siAdClips.AvailableTracks[0].TrackData)
        {
          //parse the inner XML fragment
          XElement xeClipData = XElement.Parse(Encoding.UTF8.GetString(te.EventData, 0, te.EventData.Length));
          //schedule the clip
          ssme.ScheduleClip(
              new ClipInformation
              {
                ClickThroughUrl = new Uri(xeClipData.Attribute("ClickThruUri").Value),
                ClipUrl = new Uri(xeClipData.Attribute("Uri").Value),
                IsSmoothStreamingSource = xeClipData.Attribute("Uri").Value.ToLower().Contains("ism") ? true : false,
                Duration = TimeSpan.Parse(xeClipData.Attribute("Duration").Value)
              },
              te.EventTime,
              true, //pause the timeline
              null);
        }
        //set the Clip MediaElement style
        ssme.ClipMediaElementStyle = this.Resources["ClipStyle"] as Style;


      }
    }

 

    #region INotifyPropertyChanged Members

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion
  }
}


