﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Linq; 
using System.Windows.Data;
using Microsoft.Web.Media.SmoothStreaming;

namespace SSPlayerControls
{
  [TemplatePart(Name = "IndicatorContainer", Type = typeof(Canvas))]
  public class ProgressiveDownloadIndicator : Control
  {
    public Brush DownloadedPortionBackground
    {
      get { return (Brush)GetValue(DownloadedPortionBackgroundProperty); }
      set { SetValue(DownloadedPortionBackgroundProperty, value); }
    }

    // Using a DependencyProperty as the backing store for DownloadedPortionBackground.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty DownloadedPortionBackgroundProperty =
        DependencyProperty.Register("DownloadedPortionBackground", typeof(Brush), typeof(ProgressiveDownloadIndicator), new PropertyMetadata(new SolidColorBrush(Colors.Blue)));



    public Brush DownloadingIndicatorBackground
    {
      get { return (Brush)GetValue(DownloadingIndicatorBackgroundProperty); }
      set { SetValue(DownloadingIndicatorBackgroundProperty, value); }
    }

    // Using a DependencyProperty as the backing store for DownloadingIndicatorBackground.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty DownloadingIndicatorBackgroundProperty =
        DependencyProperty.Register("DownloadingIndicatorBackground", typeof(Brush), typeof(ProgressiveDownloadIndicator), new PropertyMetadata(new SolidColorBrush(Colors.Green))); 


    public SmoothStreamingMediaElement SmoothStreamingMediaElement
    {
      get { return (SmoothStreamingMediaElement)GetValue(SmoothStreamingMediaElementProperty); }
      set { SetValue(SmoothStreamingMediaElementProperty, value); }
    }

    // Using a DependencyProperty as the backing store for TimelineItemSmoothStreamingMediaElement.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty SmoothStreamingMediaElementProperty =
        DependencyProperty.Register("SmoothStreamingMediaElement", typeof(SmoothStreamingMediaElement), typeof(ProgressiveDownloadIndicator), new PropertyMetadata(default(SmoothStreamingMediaElement), new PropertyChangedCallback(OnSmoothStreamingMediaElementChanged)));

    private static void OnSmoothStreamingMediaElementChanged(DependencyObject Source, DependencyPropertyChangedEventArgs args)
    {
      ProgressiveDownloadIndicator thisObj = Source as ProgressiveDownloadIndicator;
      thisObj.OnSmoothStreamingMediaElementChanged_Instance(args.NewValue as SmoothStreamingMediaElement);
    }
    private void OnSmoothStreamingMediaElementChanged_Instance(SmoothStreamingMediaElement medElem)
    {
      if (medElem != null)
      {
        this.SmoothStreamingMediaElement = medElem;
        if (IndicatorContainer != null) IndicatorContainer.Children.Clear();
        SmoothStreamingMediaElement.DownloadProgressChanged += new RoutedEventHandler(SmoothStreamingMediaElement_DownloadProgressChanged);
      }

    }

    private Canvas IndicatorContainer = default(Canvas);
    private Rectangle currentIndicator = default(Rectangle); 
    public ProgressiveDownloadIndicator()
    {
      base.DefaultStyleKey = typeof(ProgressiveDownloadIndicator);
    }

    public override void OnApplyTemplate()
    {
      base.OnApplyTemplate();
      IndicatorContainer = GetTemplateChild("IndicatorContainer") as Canvas;
    }

    void SmoothStreamingMediaElement_DownloadProgressChanged(object sender, RoutedEventArgs e)
    {
      AddIndicator(SmoothStreamingMediaElement);
      if (currentIndicator != null)
      {
        currentIndicator.Width = this.ActualWidth * (SmoothStreamingMediaElement.DownloadProgress - SmoothStreamingMediaElement.DownloadProgressOffset);
        if (SmoothStreamingMediaElement.DownloadProgress == 1)
          currentIndicator.Fill = DownloadedPortionBackground;
      } 
    }

    private void AddIndicator(SmoothStreamingMediaElement TimelineItemSmoothStreamingMediaElement)
    {
      if (currentIndicator != null && (double)currentIndicator.GetValue(Canvas.LeftProperty) ==
        ((SmoothStreamingMediaElement.DownloadProgressOffset == 0) ? 0 : this.ActualWidth * SmoothStreamingMediaElement.DownloadProgressOffset))      
        return;

      if (currentIndicator != null)
      {
        currentIndicator.Fill = DownloadedPortionBackground;
      }

      //check to see if there is an indicator already added for this byte range
      currentIndicator = (IndicatorContainer != null ? IndicatorContainer.Children.Where((uie) =>
        uie is Rectangle && (double)(uie as Rectangle).GetValue(Canvas.LeftProperty) == 
        ((SmoothStreamingMediaElement.DownloadProgressOffset == 0) ? 0 : this.ActualWidth * SmoothStreamingMediaElement.DownloadProgressOffset)
        ).FirstOrDefault() as Rectangle : null);
      //no indicators have been added so far
      if (currentIndicator == null && IndicatorContainer != null)
      {
        currentIndicator = new Rectangle() { Width = 0, Height = IndicatorContainer.ActualHeight, Fill = DownloadingIndicatorBackground };  
        IndicatorContainer.Children.Add(currentIndicator);
        currentIndicator.SetValue(Canvas.LeftProperty, ((SmoothStreamingMediaElement.DownloadProgressOffset == 0) ? 0 : this.ActualWidth * SmoothStreamingMediaElement.DownloadProgressOffset));
        currentIndicator.SetValue(Canvas.TopProperty, 0.0); 
      }
      

      return;

    }


  }
}



