﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Threading;
using Microsoft.Web.Media.SmoothStreaming;
using Microsoft.Web.Media;

namespace SSPlayerControls
{
  
  public class Scrubber : Slider, INotifyPropertyChanged
  {

    DispatcherTimer scrubberTimer = new DispatcherTimer();
    Thumb thumbMain = null;
    RepeatButton HorizontalTrackLargeChangeDecreaseRepeatButton = null;
    RepeatButton HorizontalTrackLargeChangeIncreaseRepeatButton = null;
    ProgressiveDownloadIndicator progressiveDownloadIndicator = null;
    TextBlock tblkSMPTE = null;
    TextBlock tblkFileTime = null;
    private SmoothStreamingMediaElement _Viewer = default(SmoothStreamingMediaElement);

    public double FrameRate
    {
      get { return (double)GetValue(FrameRateProperty); }
      set { SetValue(FrameRateProperty, value); }
    }

    // Using a DependencyProperty as the backing store for FrameRate.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty FrameRateProperty =
        DependencyProperty.Register("FrameRate", typeof(double), typeof(Scrubber), new PropertyMetadata(23D, new PropertyChangedCallback(OnFrameRateChanged)));


    private static void OnFrameRateChanged(DependencyObject source, DependencyPropertyChangedEventArgs args)
    {
      Scrubber thisObj = source as Scrubber;
      if (thisObj != null)
        thisObj.OnFrameRateChanged_Inst((double)args.NewValue);
    }

    private void OnFrameRateChanged_Inst(double FrameRate)
    {
      if(scrubberTimer != null)
        scrubberTimer.Interval = TimeSpan.FromTicks((long)(TimeCode.FromFrames(1, TimeCode.ParseFramerate(FrameRate))).TotalSecondsPrecision * 1000 * 10000); // a tick per frame;
      this.SmallChange = (double)(TimeCode.FromFrames(1, TimeCode.ParseFramerate(FrameRate))).TotalSecondsPrecision * 1000; 
    }

    public SmoothStreamingMediaElement Viewer
    {
      get
      {
        return _Viewer;
      }

      set
      {
        if (value != _Viewer)
        {
          _Viewer = value;
          if (_Viewer != null)
          {
            if (progressiveDownloadIndicator != null) 
              progressiveDownloadIndicator.SmoothStreamingMediaElement = _Viewer;

            OnFrameRateChanged_Inst(FrameRate);

            _Viewer.MediaOpened += new RoutedEventHandler((s, e) =>
            {
              this.Minimum = 0;
              this.Maximum = _Viewer.NaturalDuration.TimeSpan.TotalMilliseconds;
            });

            _Viewer.CurrentStateChanged += new RoutedEventHandler((sender, args) =>
            {
              switch (_Viewer.CurrentState)
              {

                case SmoothStreamingMediaElementState.Playing:
                  scrubberTimer.Start();
                  break;
                default: 
                  break;
              }
            });
          }
          if (PropertyChanged != null)
            PropertyChanged(this, new PropertyChangedEventArgs("Viewer"));
        }

      }
    }


    public Scrubber()
    {
      base.DefaultStyleKey = typeof(Scrubber);
    }
    public override void OnApplyTemplate()
    {
      base.OnApplyTemplate();
      thumbMain = GetTemplateChild("HorizontalThumb") as Thumb;
      progressiveDownloadIndicator = GetTemplateChild("progressiveDownloadIndicator") as ProgressiveDownloadIndicator;
      HorizontalTrackLargeChangeDecreaseRepeatButton = GetTemplateChild("HorizontalTrackLargeChangeDecreaseRepeatButton") as RepeatButton;
      HorizontalTrackLargeChangeIncreaseRepeatButton = GetTemplateChild("HorizontalTrackLargeChangeIncreaseRepeatButton") as RepeatButton;
      tblkSMPTE = GetTemplateChild("tblkSMPTE") as TextBlock;
      tblkFileTime = GetTemplateChild("tblkFileTime") as TextBlock;

      if (progressiveDownloadIndicator != null)
        progressiveDownloadIndicator.SmoothStreamingMediaElement = Viewer;

      
      if (thumbMain != null)
      {
        thumbMain.DragStarted += new DragStartedEventHandler((sender, args) =>
        {
          if (Viewer.CurrentState == SmoothStreamingMediaElementState.Playing)
          {
            if (scrubberTimer.IsEnabled) 
              scrubberTimer.Stop();
            if (Viewer.CurrentState != SmoothStreamingMediaElementState.Paused) 
              Viewer.Pause();
            //Viewer.Position = TimeSpan.FromMilliseconds(this.Value);
          }
        });
      } 
      scrubberTimer.Tick += new EventHandler((sender, args) =>
      {
        if (Viewer != null) 
          this.Value = Viewer.Position.TotalMilliseconds;
      });
    }



    protected override void OnValueChanged(double oldValue, double newValue)
    {

      if (Viewer != null && Viewer.CurrentState != SmoothStreamingMediaElementState.Playing)
      {
        Viewer.Position = TimeSpan.FromMilliseconds(this.Value);
      }

      UpdateOnScreenPositionInfo((long)(this.Value * 10000));
      base.OnValueChanged(oldValue, newValue); 
    }

    private void UpdateOnScreenPositionInfo(long Position)
    {
      if (tblkSMPTE != null)
        //timecode ranges ?
        tblkSMPTE.Text = TimeCode.FromTicks(Position, TimeCode.ParseFramerate(FrameRate)).ToString();
      if (tblkFileTime != null)
        //timecode ranges ?
        tblkFileTime.Text = TimeSpan.FromTicks(Position).ToString();

      System.Diagnostics.Debug.WriteLine("IsLive={0},IsLivePosition={1},LivePlaybackStartPosition={2},LiveBufferSize={3},LivePosition={4},Position={5},TrailingWindowDuration={6}", Viewer.IsLive, Viewer.IsLivePosition, Viewer.LivePlaybackStartPosition, Viewer.LiveBufferSize, Viewer.LivePosition,Viewer.Position.TotalSeconds,Viewer.TrailingWindowDuration);

    }

    #region INotifyPropertyChanged Members

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion
  }
}
