﻿// <copyright file="Converters.cs" company="Microsoft Corporation">
//     Copyright (c) Microsoft Corporation. All rights reserved.
// </copyright>

using System;
using System.Text;
using System.Windows;
using System.Windows.Data;

namespace SSPlayerControls
{
  public class StringToUriConverter : IValueConverter
  {
    public object Convert(object value, Type targetType,
      object parameter, System.Globalization.CultureInfo culture)
    {
      //check to see that the parameter types are conformant
      if (value.GetType() != typeof(string) || targetType != typeof(Uri))
        return null;

      return new Uri(value as String);
    }

    public object ConvertBack(object value, Type targetType,
      object parameter, System.Globalization.CultureInfo culture)
    {
      throw new NotImplementedException();
    }
  }
  
  
  public class MillisecondsToTimeStringConverter : IValueConverter
  {
    public object Convert(object value, Type targetType,
      object parameter, System.Globalization.CultureInfo culture)
    {
      //check to see that the parameter types are conformant
      if (value.GetType() != typeof(double) || targetType != typeof(string))
        return null;
       
      TimeSpan src = new TimeSpan((long)(10000 * (double)value));
      //translate
      if( parameter != null && parameter is string)
      {
        if((string)parameter == "nounits")
        {
          return string.Format("{0:00}:{1:00}:{2:00}.{3:000}", src.Hours, src.Minutes, src.Seconds, src.Milliseconds);
        }
        else if ((string)parameter == "short")
        {
          StringBuilder formatString = new StringBuilder();
          if (src.Hours != 0) formatString.AppendFormat("{0:#0h } ", src.Hours);
          if (src.Minutes != 0) formatString.AppendFormat("{0:#0m } ", src.Minutes);
          formatString.AppendFormat("{0:#0s} {1:##0ms}", src.Seconds,src.Milliseconds);
          return formatString.ToString();
        }
        else
          return string.Format("{0:00}h {1:00}m {2:00}s {3:000}ms", src.Hours, src.Minutes, src.Seconds, src.Milliseconds);
      }
      else
        return string.Format("{0:00}h {1:00}m {2:00}s {3:000}ms", src.Hours, src.Minutes, src.Seconds, src.Milliseconds);
      
    }

    public object ConvertBack(object value, Type targetType,
      object parameter, System.Globalization.CultureInfo culture)
    {
      throw new NotImplementedException();
    }
  }

  public class BoolToVisibilityConverter : IValueConverter
  {
    public object Convert(object value, Type targetType,
      object parameter, System.Globalization.CultureInfo culture)
    {
      //check to see that the parameter types are conformant
      if (!(value is bool) || targetType != typeof(Visibility))
        return null;
      if(parameter != null && parameter is string && (parameter as string).ToLower().Equals("reverse"))
        return (bool)value == false ? Visibility.Visible : Visibility.Collapsed;
      else
        return (bool)value == true ? Visibility.Visible : Visibility.Collapsed;

    }


    public object ConvertBack(object value, Type targetType,
      object parameter, System.Globalization.CultureInfo culture)
    {
      throw new NotImplementedException();
    }
  }
  public class BoolToOpacityConverter : IValueConverter
  {
    public object Convert(object value, Type targetType,
      object parameter, System.Globalization.CultureInfo culture)
    {
      //check to see that the parameter types are conformant
      if (!(value is bool) || targetType != typeof(double))
        return null;

      if (parameter != null && parameter is string && (parameter as string).ToLower().Equals("reverse"))
        return (bool)value == false ? 1 : 0;
      else
        return (bool)value == true ? 1 : 0; 

    }


    public object ConvertBack(object value, Type targetType,
      object parameter, System.Globalization.CultureInfo culture)
    {
      throw new NotImplementedException();
    }
  }

   
}
