﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using Microsoft.Web.Media.SmoothStreaming;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.Windows.Markup;
using System.Globalization;
using System.Collections.Generic;
using System.Windows.Browser;

namespace SSPlayerControls
{
  
  public class PlayerUI : Control, INotifyPropertyChanged
  {
    Scrubber scrubber = null;
    ButtonsPanel buttonsPanel = null;
    Grid MediaElementContainer = null;
    internal SmoothStreamingMediaElement ssme = null; 



    public double FrameRate
    {
      get { return (double)GetValue(FrameRateProperty); }
      set { SetValue(FrameRateProperty, value); }
    }

    // Using a DependencyProperty as the backing store for FrameRate.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty FrameRateProperty =
        DependencyProperty.Register("FrameRate", typeof(double), typeof(PlayerUI), new PropertyMetadata(23D));
    public string SmoothSource
    {
      get { return (string)GetValue(SmoothSourceProperty); }
      set { SetValue(SmoothSourceProperty, value); }
    }

    // Using a DependencyProperty as the backing store for SmoothSource.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty SmoothSourceProperty =
        DependencyProperty.Register("SmoothSource", typeof(string), typeof(PlayerUI), new PropertyMetadata(null, new PropertyChangedCallback(OnSmoothSourceChanged)));

    //Change handler for dependency property SmoothSourceProperty
    private static void OnSmoothSourceChanged(DependencyObject Src, DependencyPropertyChangedEventArgs Args)
    {
      PlayerUI thisObj = Src as PlayerUI;
      //act on the change...
      if (thisObj.ssme != null && Uri.IsWellFormedUriString(Args.NewValue as string, UriKind.Absolute))
      {
        thisObj.SetMediaSource(Args.NewValue as string, thisObj.ssme);
      }
    }

    private void SetMediaSource(string MediaSourceUri,
      SmoothStreamingMediaElement ssme)
    {

      if (MediaSourceUri.Contains(".ism"))
        ssme.SmoothStreamingSource = new Uri(MediaSourceUri);
      else
        ssme.Source = new Uri(MediaSourceUri);
    }  

   

    public PlayerUI()
    {
      base.DefaultStyleKey = typeof(PlayerUI);
    }

    public override void OnApplyTemplate()
    {
      base.OnApplyTemplate();
      ssme = GetTemplateChild("ssme") as SmoothStreamingMediaElement;
      scrubber = GetTemplateChild("scrubber") as Scrubber;
      buttonsPanel = GetTemplateChild("buttonsPanel") as ButtonsPanel; 

      MediaElementContainer = GetTemplateChild("MediaElementContainer") as Grid;
      if (scrubber != null && ssme != null)
        scrubber.Viewer = ssme;
      if (buttonsPanel != null && ssme != null)
      {
        buttonsPanel.Viewer = ssme;
        //the position change actually happens in the Scrubber class(Scrubber.cs) - line 201
        buttonsPanel.FrameBackward += new EventHandler((s, e) =>
        {
          scrubber.Value -= (double)((TimeCode.FromFrames(1, TimeCode.ParseFramerate(FrameRate))).TotalSecondsPrecision * 1000); // - you could also try this - has the same effect
        });
        buttonsPanel.FrameForward += new EventHandler((s, e) =>
        {
          scrubber.Value += (double)((TimeCode.FromFrames(1, TimeCode.ParseFramerate(FrameRate))).TotalSecondsPrecision * 1000);// - you could also try this - has the same effect
        });

      }

      if (ssme != null)
      {

        ssme.ConfigPath = "config.xml";

        ssme.MediaOpened += new RoutedEventHandler((sender, args) =>
        {
          /*
           * SAMPLE CODE - Illustrating track selection
          if (subscriber.AccessLevel != "Premium")
          {

            StreamInfo videoStream = ssme.GetStreamInfoForStreamType("video");
            List<TrackInfo> allowedTracks = videoStream.AvailableTracks.Where((ti) => ti.Bitrate < 1000000).ToList();
            ssme.SelectTracksForStream(videoStream, allowedTracks, false);
          } 
           */ 
           
          ssme.Position = TimeSpan.FromTicks((long)(0));

          AddAndHandleMarkers();
        });

        ssme.CurrentStateChanged += new RoutedEventHandler((s, e) =>
        {
          return;
        });
        
        ssme.ManifestReady += new EventHandler<EventArgs>((sender, args) =>
        { 
         
          return;
        });

        ssme.MediaFailed += new EventHandler<ExceptionRoutedEventArgs>((s, e) =>
        {
          return;
        });

        ssme.ManifestMerge +=
          new SmoothStreamingMediaElement.ManifestMergeHandler((sender) =>
          {
            object ParsedExternalManifest = null;
            //URI of the right external manifest based on current locale
            //for example expands to 
            string UriString = string.Format("http://localhost/SmoothStreaming/Media/FighterPilot/{0}/CC.xml",
              CultureInfo.CurrentCulture.Name);
            //parse the external manifest - time out in 3 secs
            ssme.ParseExternalManifest(new Uri(UriString), 3000, out ParsedExternalManifest);
            //merge the external manifest
            ssme.MergeExternalManifest(ParsedExternalManifest); 
          });
       

 
      }


    }

    

    private void AddAndHandleMarkers()
    {

      StreamInfo CCStream = ssme.AvailableStreams.Where((si) => si.Name.ToLower() == "closedcaptions" && si.Subtype.ToLower() == "capt").FirstOrDefault();
      StreamInfo AnimStream = ssme.AvailableStreams.Where((si) => si.Name.ToLower() == "animations" && si.Subtype.ToLower() == "data").FirstOrDefault();
      foreach (TimelineEvent te in CCStream.AvailableTracks[0].TrackData)
      {
        TimelineMarker tm = new TimelineMarker() { Text = Encoding.UTF8.GetString(te.EventData, 0, te.EventData.Length), Time = te.EventTime, Type = "CC" };
        ssme.Markers.Add(tm);
      }
      foreach (TimelineEvent te in AnimStream.AvailableTracks[0].TrackData)
      {
        TimelineMarker tm = new TimelineMarker() { Text = Encoding.UTF8.GetString(te.EventData, 0, te.EventData.Length), Time = te.EventTime, Type = "ANIM" };
        ssme.Markers.Add(tm);
      }
      
      ssme.MarkerReached += new TimelineMarkerRoutedEventHandler((s, e) =>
      {
        XElement xElem = XElement.Parse(e.Marker.Text);

        //if closed caption event
        if (e.Marker.Type == "CC")
        {

          //if we are adding a caption
          if (xElem.Attribute("Action") != null &&
            xElem.Attribute("Action").Value == "ADD")
          {
            //add a TextBlock 
            MediaElementContainer.Children.Add(new TextBlock()
            {
              Name = xElem.Attribute("Id").Value,
              Text = xElem.Value,
              HorizontalAlignment = HorizontalAlignment.Center,
              VerticalAlignment = VerticalAlignment.Bottom,
              Margin = new Thickness(0, 0, 0, 20),
              Foreground = new SolidColorBrush(Colors.White),
              FontSize = 22
            });
          }
          //if we are removing a caption
          else if (xElem.Attribute("Action") != null &&
            xElem.Attribute("Action").Value == "REMOVE")
          {
            //remove the TextBlock
            MediaElementContainer.Children.Remove(MediaElementContainer.Children.
              Where((uie) => uie is FrameworkElement &&
                (uie as FrameworkElement).Name == (xElem.Attribute("Id").Value)).
                FirstOrDefault() as UIElement);
          }
        }
        //if animation event
        else if (e.Marker.Type == "ANIM")
        {

          //if we are adding the animation
          if (xElem.Attribute("Action") != null &&
            xElem.Attribute("Action").Value == "ADD")
          {
            //add the animation to a grid overlay
            Grid AnimationContainer = XamlReader.Load(xElem.Element(
              XName.Get("Grid", "http://schemas.microsoft.com/winfx/2006/xaml/presentation")).
              ToString(SaveOptions.DisableFormatting)) as Grid;
            AnimationContainer.Name = xElem.Attribute("Id").Value;
            MediaElementContainer.Children.Add(AnimationContainer);
            //load the animation
            Storyboard anim = AnimationContainer.Resources["SceneAnimation"] as
              Storyboard;
            //handle animation completion
            anim.Completed += new EventHandler((animSender, animargs) =>
            {
              //if animation is completed, remove it
              MediaElementContainer.Children.Remove(MediaElementContainer.Children.
                Where((uie) => uie is FrameworkElement && (uie as FrameworkElement).
                  Name == (xElem.Attribute("Id").Value)).
                  FirstOrDefault() as UIElement);
            });
            //start the animation
            anim.Begin();
          }

        }
      });
    }

 

    #region INotifyPropertyChanged Members

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion
  }
}


